@@ -13,6 +13,7 @@
 """Here is defined the Expr class."""
 
 from __future__ import print_function
+from __future__ import absolute_import
 import sys
 import warnings
 
@@ -20,10 +21,11 @@ import numpy as np
 import tables as tb
 from numexpr.necompiler import getContext, getExprNames, getType, NumExpr
 from numexpr.expressions import functions as numexpr_functions
-from tables.exceptions import PerformanceWarning
-from tables.parameters import IO_BUFFER_SIZE, BUFFER_TIMES
-
-from tables._past import previous_api
+from .exceptions import PerformanceWarning
+from .parameters import IO_BUFFER_SIZE, BUFFER_TIMES
+import six
+from six.moves import range
+from six.moves import zip
 
 
 class Expr(object):
@@ -37,6 +39,12 @@ class Expr(object):
     but in addition to NumPy objects, it also accepts disk-based homogeneous
     arrays, like the Array, CArray, EArray and Column PyTables objects.
 
+    .. warning::
+
+        Expr class only offers a subset of the Numexpr features due to the
+        complexity of implement some of them when dealing with huge amount of
+        data.
+
     All the internal computations are performed via the Numexpr package,
     so all the broadcast and upcasting rules of Numexpr applies here too.
     These rules are very similar to the NumPy ones, but with some exceptions
@@ -45,6 +53,7 @@ class Expr(object):
     constructor and methods as well as that of Numexpr, if you want to fully
     grasp these particularities.
 
+
     Parameters
     ----------
     expr : str
@@ -178,8 +187,8 @@ class Expr(object):
         self.names, _ = getExprNames(expr, context)
 
         # Raise a ValueError in case we have unsupported objects
-        for name, var in vars_.items():
-            if type(var) in (int, int, float, str):
+        for name, var in six.iteritems(vars_):
+            if type(var) in (int, float, str):
                 continue
             if not isinstance(var, (tb.Leaf, tb.Column)):
                 if hasattr(var, "dtype"):
@@ -193,7 +202,7 @@ class Expr(object):
         # NumPy arrays to be copied? (we don't need to worry about
         # PyTables objects, as the reads always return contiguous and
         # aligned objects, or at least I think so).
-        for name, var in vars_.items():
+        for name, var in six.iteritems(vars_):
             if isinstance(var, np.ndarray):
                 # See numexpr.necompiler.evaluate for a rational
                 # of the code below
@@ -255,7 +264,7 @@ class Expr(object):
 
         # Get the names of variables used in the expression.
         exprvars_cache = self._exprvars_cache
-        if not expression in exprvars_cache:
+        if expression not in exprvars_cache:
             # Protection against growing the cache too much
             if len(exprvars_cache) > 256:
                 # Remove 10 (arbitrary) elements from the cache
@@ -308,8 +317,6 @@ class Expr(object):
             reqvars[var] = val
         return reqvars
 
-    _requiredExprVars = previous_api(_required_expr_vars)
-
     def set_inputs_range(self, start=None, stop=None, step=None):
         """Define a range for all inputs in expression.
 
@@ -326,8 +333,6 @@ class Expr(object):
         self.stop = stop
         self.step = step
 
-    setInputsRange = previous_api(set_inputs_range)
-
     def set_output(self, out, append_mode=False):
         """Set out as container for output as well as the append_mode.
 
@@ -360,8 +365,6 @@ class Expr(object):
                 "with an `append()` method (like the `EArray`)")
         self.append_mode = append_mode
 
-    setOutput = previous_api(set_output)
-
     def set_output_range(self, start=None, stop=None, step=None):
         """Define a range for user-provided output object.
 
@@ -379,8 +382,6 @@ class Expr(object):
         self.o_stop = stop
         self.o_step = step
 
-    setOutputRange = previous_api(set_output_range)
-
     # Although the next code is similar to the method in `Leaf`, it
     # allows the use of pure NumPy objects.
     def _calc_nrowsinbuf(self, object_):
@@ -474,7 +475,7 @@ value of dimensions that are orthogonal (and preferably close) to the
             (start, stop, step) = slice(
                 self.start, self.stop, self.step).indices(shape[maindim])
             shape[maindim] = min(
-                shape[maindim], len(range(0, stop - start, step)))
+                shape[maindim], len(range(start, stop, step)))
             i_nrows = shape[maindim]
         else:
             start, stop, step = 0, 0, None
@@ -584,7 +585,10 @@ value of dimensions that are orthogonal (and preferably close) to the
 
         if i_nrows == 0:
             # No elements to compute
-            return self._single_row_out
+            if start >= stop and self.start is not None:
+                return out
+            else:
+                return self._single_row_out
 
         # Create a key that selects every element in inputs and output
         # (including the main dimension)
